﻿using System.Windows;
using System.Windows.Controls;
using CommunityToolkit.Mvvm.ComponentModel;
using LibS7Adv;
using CtrEditor.Simulacion;
using CtrEditor.FuncionesBase;
using System.ComponentModel;
using System.Numerics;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucGuia.xaml
    /// </summary>
    public partial class osGuia : osBase, IosBase
    {
        private simGuia SimGeometria;

        public static string NombreClase()
        {
            return "Guía";
        }
        private string nombre = NombreClase();

        [property: Category("Identificación")]
        [property: Description("Nombre identificativo del objeto")]
        [property: Name("Nombre")]
        public override string Nombre
        {
            get => nombre;
            set => SetProperty(ref nombre, value);
        }

        [ObservableProperty]
        [property: Category("Configuración")]
        [property: Description("Grosor de la guía en metros")]
        [property: Name("Grosor de la Guía")]
        public float altoGuia;



        private void ActualizarGeometrias()
        {
            if (_visualRepresentation is ucGuia uc && SimGeometria != null)
            {
                var topLeft = new Vector2(Left, Top);
                
                // ✅ SISTEMA INTELIGENTE: Solo recrear si las dimensiones han cambiado
                if (HasGuideDimensionsChanged(SimGeometria, Ancho, AltoGuia))
                {
                    //System.Diagnostics.Debug.WriteLine($"[osGuia] Recreando guía por cambio de dimensiones: {Ancho}x{AltoGuia}");
                    
                    // ✅ RECREAR COMPLETAMENTE: Las dimensiones cambiaron
                    SimGeometria.Create(Ancho, AltoGuia, topLeft, Angulo);
                    SimGeometria.SetDimensions(Ancho, AltoGuia);
                }
                else
                {
                   // System.Diagnostics.Debug.WriteLine($"[osGuia] Solo actualizando posición/rotación: Left={Left}, Top={Top}, Angulo={Angulo}");
                    
                    // ✅ SOLO ACTUALIZAR POSICIÓN/ROTACIÓN: Usar dimensiones reales para conversión correcta
                    SimGeometria.UpdateFromWpfParameters(topLeft, Angulo, Ancho, AltoGuia);
                }
            }
        }

        public override void OnMoveResizeRotate()
        {
            ActualizarGeometrias();
        }

        public osGuia()
        {
            Ancho = 1;
            AltoGuia = 0.03f;
        }

        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion
            ActualizarGeometrias();
        }
        public override void UpdateControl(int elapsedMilliseconds)
        {
        }
        public override void UpdateGeometryStep()
        {
        }
        public override void UpdatePLC(PLCViewModel plc, int elapsedMilliseconds) { }
        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            base.ucLoaded();

            if (_visualRepresentation is ucGuia uc)
                SimGeometria = AddLine(simulationManager, uc.Guia);
        }
        public override void ucUnLoaded()
        {
            // El UserControl se esta eliminando
            // eliminar el objeto de simulacion
            simulationManager?.Remove(SimGeometria);
        }

        public override void AnchoChanged(float value)
        {
            ActualizarGeometrias();
        }

        public override void AnguloChanged(float value)
        {
            ActualizarGeometrias();
        }

        // Método llamado cuando cambia AltoGuia
        partial void OnAltoGuiaChanged(float value)
        {
            ActualizarGeometrias();
        }
    }

    public partial class ucGuia : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }
        public int zIndex_fromFrames { get; set; }

        public ucGuia()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
            this.Unloaded += OnUnloaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        private void OnUnloaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucUnLoaded();
        }
        public void Highlight(bool State) { }
        public ZIndexEnum ZIndex_Base()
        {
            return ZIndexEnum.Guias;
        }

    }
}
