﻿using LibS7Adv;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using CommunityToolkit.Mvvm.ComponentModel;
using Newtonsoft.Json.Linq;
using CtrEditor.FuncionesBase;
using System.ComponentModel;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucAnalogTag.xaml
    /// </summary>
    /// 

    public partial class osAnalogTag : osBase, IosBase
    {

        // Otros datos y métodos relevantes para la simulación

        public static string NombreClase()
        {
            return "Tag Analógico";
        }
        private string nombre = NombreClase();

        [property: Category("Identificación")]
        [property: Description("Nombre identificativo del objeto")]
        [property: Name("Nombre")]
        public override string Nombre
        {
            get => nombre;
            set => SetProperty(ref nombre, value);
        }

        [ObservableProperty]
        [property: Category("Apariencia")]
        [property: Description("Tamaño visual del tag")]
        [property: Name("Tamaño")]
        public float tamano;

        public override void OnResize(float Delta_Width, float Delta_Height)
        {
            Tamano += Delta_Width + Delta_Height;
        }

        [ObservableProperty]
        [NotifyPropertyChangedFor(nameof(Value))]
        [property: Category("Enlace PLC")]
        [property: Description("Dirección del tag en el PLC")]
        [property: Name("Tag")]
        public string tag;

        [ObservableProperty]
        [property: Category("Identificación")]
        [property: Description("Descripción del tag")]
        [property: Name("Descripción")]
        public string descripcion;

        [ObservableProperty]
        [NotifyPropertyChangedFor(nameof(Value))]
        [property: Category("Configuración")]
        [property: Description("Valor mínimo de entrada escalado")]
        [property: Name("Mínimo Entrada")]
        public float min_IN_Scaled;

        [ObservableProperty]
        [NotifyPropertyChangedFor(nameof(Value))]
        [property: Category("Configuración")]
        [property: Description("Valor máximo de entrada escalado")]
        [property: Name("Máximo Entrada")]
        public float max_IN_Scaled;

        [ObservableProperty]
        [NotifyPropertyChangedFor(nameof(Value))]
        [property: Category("Configuración")]
        [property: Description("Valor mínimo de salida escalado")]
        [property: Name("Mínimo Salida")]
        public float min_OUT_Scaled;

        [ObservableProperty]
        [NotifyPropertyChangedFor(nameof(Value))]
        [property: Category("Configuración")]
        [property: Description("Valor máximo de salida escalado")]
        [property: Name("Máximo Salida")]
        public float max_OUT_Scaled;

        [ObservableProperty]
        [property: Category("Simulación")]
        [property: Description("Valor actual del tag")]
        [property: Name("Valor")]
        public float value;

        partial void OnValueChanged(float value)
        {
            EscribirWordTagScaled(Tag, Value, Min_IN_Scaled, Max_IN_Scaled, Min_OUT_Scaled, Max_OUT_Scaled);
        }

        public osAnalogTag()
        {
            Tamano = 0.30f;
            tag = "%MW50.0";
            Descripcion = "Nombre del Tag:";
            Max_OUT_Scaled = 27648;
            Min_OUT_Scaled = 0;
            min_IN_Scaled = 0;
            max_IN_Scaled = 100;

        }

        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion

        }
        public override void UpdatePLC(PLCViewModel plc, int elapsedMilliseconds)
        {
        }

        public override void UpdatePLCPrimerCiclo()
        {
            // Escribimos el valor actual al iniciar la conexion
            OnValueChanged(Value);
        }

        public override void UpdateControl(int elapsedMilliseconds)
        {
            // Calculamos la velocidad

        }

        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            base.ucLoaded();

        }
    }

    public partial class ucAnalogTag : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }
        public int zIndex_fromFrames { get; set; }

        public ucAnalogTag()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
            this.Unloaded += OnUnloaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        private void OnUnloaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucUnLoaded();
        }
        public void Highlight(bool State) { }
        public ZIndexEnum ZIndex_Base()
        {
            return ZIndexEnum.Estaticos;
        }
    }
}
